<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/31/2008 *
 * ===============                              Last change: 08/31/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : surfbar_functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Functions for surfbar                            *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer die Surfbar                      *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (ereg(basename(__FILE__), $_SERVER['PHP_SELF'])) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

// Admin has added an URL with given user id
function SURFBAR_ADMIN_ADD_URL ($url, $uid, $reward, $costs, $paymentId) {
	// Do some pre-checks
	if (!IS_ADMIN()) {
		// Not an admin
		return false;
	} elseif (!VALIDATE_URL($url)) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, $uid)) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS($uid)) {
		// No more allowed!
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, $uid, $reward, $costs, $paymentId, "CONFIRMED", "unlock");
}
// Member has added an URL
function SURFBAR_MEMBER_ADD_URL ($url) {
	global $_CONFIG;

	// Do some pre-checks
	if (!IS_LOGGED_IN()) {
		// Not a member
		return false;
	} elseif (!VALIDATE_URL($url)) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, $GLOBALS['userid'])) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS()) {
		// No more allowed!
		return false;
	}

	// Do we have fixed or dynamic payment model?
	$reward = SURFBAR_DETERMINE_REWARD();
	$costs  = SURFBAR_DETERMINE_COSTS();

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, $GLOBALS['userid'], $reward, $costs);
}
// Looks up by an URL
function SURFBAR_LOOKUP_BY_URL ($url) {
	// Now lookup that given URL by itself
	$urlArray = SURFBAR_GET_URL_DATA($url, "url");

	// Was it found?
	return (count($urlArray) > 0);
}
// Load URL data by given search term and column
function SURFBAR_GET_URL_DATA ($searchTerm, $column="id", $order="id", $sort="ASC", $group="id") {
	global $lastUrlData;

	// By default nothing is found
	$lastUrlData = array();

	// Is the column an id number?
	if (($column == "id") || ($column == "userid")) {
		// Extra secure input
		$searchTerm = bigintval($searchTerm);
	} // END - if

	// Look up the record
	$result = SQL_QUERY_ESC("SELECT id, userid, url, reward, costs, views_total, status, registered, last_locked, lock_reason
FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE %s='%s'
ORDER BY %s %s",
		array($column, $searchTerm, $order, $sort), __FILE__, __LINE__);

	// Is there at least one record?
	if (SQL_NUMROWS($result) > 0) {
		// Then load all!
		while ($dataRow = SQL_FETCHARRAY($result)) {
			// Shall we group these results?
			if ($group == "id") {
				// Add the row by id as index
				$lastUrlData[$dataRow['id']] = $dataRow;
			} else {
				// Group entries
				$lastUrlData[$dataRow[$group]][$dataRow['id']] = $dataRow;
			}
		} // END - while
	} // END - if

	// Free the result
	SQL_FREERESULT($result);

	// Return the result
	return $lastUrlData;
}
// Registers an URL with the surfbar. You should have called SURFBAR_LOOKUP_BY_URL() first!
function SURFBAR_REGISTER_URL ($url, $uid, $reward, $costs, $paymentId=0, $status="PENDING", $addMode="reg") {
	global $_CONFIG;

	// Make sure by the user registered URLs are always pending
	if ($addMode == "reg") $status = "PENDING";

	// Prepare content
	$content = array(
		'url'         => $url,
		'frametester' => FRAMETESTER($url),
		'uid'         => $uid,
		'reward'      => $reward,
		'costs'       => $costs,
		'payment_id'  => $paymentId,
		'status'      => $status
	);

	// Insert the URL into database
	$content['insert_id'] = SURFBAR_INSERT_URL_BY_ARRAY($content);

	// Translate status, reward and costs
	$content['status'] = SURFBAR_TRANSLATE_STATUS($content['status']);
	$content['reward'] = TRANSLATE_COMMA($content['reward']);
	$content['costs']  = TRANSLATE_COMMA($content['costs']);

	// If in reg-mode we notify admin
	if (($addMode == "reg") || ($_CONFIG['surfbar_notify_admin_unlock'] == "Y")) {
		// Notify admin even when he as unlocked an email
		SURFBAR_NOTIFY_ADMIN("url_{$addMode}", $content);
	} // END - if

	// Send mail to user
	SURFBAR_NOTIFY_USER("url_{$addMode}", $content);

	// Return the insert id
	return $content['insert_id'];
}
// Inserts an url by given data array and return the insert id
function SURFBAR_INSERT_URL_BY_ARRAY ($urlData) {
	// Just run the insert query for now
	SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_urls (userid, url, reward, costs, payment_id, status) VALUES(%s, '%s', %s, %s, %d, '%s')",
		array(
			bigintval($urlData['uid']),
			$urlData['url'],
			(float)$urlData['reward'],
			(float)$urlData['costs'],
			bigintval($urlData['payment_id']),
			$urlData['status']
		), __FILE__, __LINE__
	);

	// Return insert id
	return SQL_INSERTID();
}
// Notify admin(s) with a selected message and content
function SURFBAR_NOTIFY_ADMIN ($messageType, $content) {
	// Prepare template name
	$templateName = sprintf("admin_surfbar_%s", $messageType);

	// Prepare subject
	$eval = sprintf("\$subject = ADMIN_SURFBAR_NOTIFY_%s_SUBJECT;",
		strtoupper($messageType)
	);
	eval($eval);

	// Send the notification out
	SEND_ADMIN_NOTIFICATION($subject, $templateName, $content, $content['uid']);
}
// Notify the user about the performed action
function SURFBAR_NOTIFY_USER ($messageType, $content) {
	// Prepare template name
	$templateName = sprintf("member_surfbar_%s", $messageType);

	// Prepare subject
	$eval = sprintf("\$subject = MEMBER_SURFBAR_NOTIFY_%s_SUBJECT;",
		strtoupper($messageType)
	);
	eval($eval);

	// Load template
	$mailText = LOAD_EMAIL_TEMPLATE($templateName, $content);

	// Send the email
	SEND_EMAIL($content['uid'], $subject, $mailText);
}
// Translate the URL status
function SURFBAR_TRANSLATE_STATUS ($status) {
	// Create constant name
	$constantName = sprintf("SURFBAR_URL_STATUS_%s", strtoupper($status));

	// Set default translated status
	$statusTranslated = "!".$constantName."!";

	// Generate eval() command
	if (defined($constantName)) {
		$eval = "\$statusTranslated = ".$constantName.";";
		eval($eval);
	} // END - if

	// Return result
	return $statusTranslated;
}
// Determine reward
function SURFBAR_DETERMINE_REWARD () {
	global $_CONFIG;

	// Do we have static or dynamic?
	if ($_CONFIG['surfbar_pay_model'] == "STATIC") {
		// Static model, so choose static values
		$reward = $_CONFIG['surfbar_static_reward'];
	} else {
		// Dynamic model, so calculate values
		die("DYNAMIC payment model not yet supported!");
	}

	// Return reward
	return $reward;
}
// Determine costs
function SURFBAR_DETERMINE_COSTS () {
	global $_CONFIG;

	// Do we have static or dynamic?
	if ($_CONFIG['surfbar_pay_model'] == "STATIC") {
		$costs  = $_CONFIG['surfbar_static_costs'];
	} else {
		// Dynamic model, so calculate values
		die("DYNAMIC payment model not yet supported!");
	}

	// Return costs
	return $costs;
}
// Determine right template name
function SURFBAR_DETERMINE_TEMPLATE_NAME() {
	// Default is the frameset
	$templateName = "surfbar_frameset";

	// Any frame set? ;-)
	if (isset($_GET['frame'])) {
		// Use the frame as a template name part... ;-)
		$templateName = sprintf("surfbar_frame_%s",
			SQL_ESCAPE($_GET['frame'])
		);
	} // END - if

	// Return result
	return $templateName;
}
// Check if the "reload lock" of the current user is full, call this function
// before you call SURFBAR_CHECK_RELOAD_LOCK().
function SURFBAR_CHECK_RELOAD_FULL() {
	global $SURFBAR_CACHE, $_CONFIG;

	// Default is full!
	$isFull = true;

	// Do we have static or dynamic mode?
	if ($_CONFIG['surfbar_pay_model'] == "STATIC") {
		// Cache static reload lock
		$SURFBAR_CACHE['surf_lock'] = $_CONFIG['surfbar_static_lock'];
		//DEBUG_LOG(__FUNCTION__.":Fixed surf lock is ".$_CONFIG['surfbar_static_lock']."");

		// Ask the database
		$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt FROM "._MYSQL_PREFIX."_surfbar_locks
WHERE userid=%s AND (UNIX_TIMESTAMP() - ".SURFBAR_GET_DATA('surf_lock').") < UNIX_TIMESTAMP(last_surfed)
LIMIT 1",
			array($GLOBALS['userid']), __FILE__, __LINE__
		);

		// Fetch row
		list($SURFBAR_CACHE['user_locks']) = SQL_FETCHROW($result);

		// Is it null?
		if (is_null($SURFBAR_CACHE['user_locks'])) {
			// Then fix it to zero!
			$SURFBAR_CACHE['user_locks'] = 0;
		} // END - if

		// Free result
		SQL_FREERESULT($result);

		// Get total URLs
		$total = SURFBAR_GET_TOTAL_URLS();

		// Do we have some URLs in lock? Admins can always surf on own URLs!
		//DEBUG_LOG(__FUNCTION__.":userLocks=".SURFBAR_GET_DATA('user_locks').",total={$total}");
		$isFull = ((SURFBAR_GET_DATA('user_locks') == $total) && ($total > 0));
	} else {
		// Dynamic model...
		die("DYNAMIC not yet implemented!");
	}

	// Return result
	return $isFull;
}
// Get total amount of URLs of given status for current user or of CONFIRMED URLs by default
function SURFBAR_GET_TOTAL_URLS ($status="CONFIRMED") {
	// Determine depleted user account
	$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS();

	// Get amount from database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE userid NOT IN (".implode(",", $UIDs).") AND status='%s'",
		array($status), __FILE__, __LINE__
	);

	// Fetch row
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}
// Check wether the user is allowed to book more URLs
function SURFBAR_IF_USER_BOOK_MORE_URLS ($uid=0) {
	global $_CONFIG;

	// Simply check it out
	return (SURFBAR_GET_TOTAL_USER_URLS($uid) < $_CONFIG['surfbar_max_order']);
}
// Get total amount of URLs of given status for current user
function SURFBAR_GET_TOTAL_USER_URLS ($uid=0) {
	global $_CONFIG;

	// Is the user 0 and user is logged in?
	if (($uid == 0) && (IS_LOGGED_IN())) {
		// Then use this userid
		$uid = $GLOBALS['userid'];
	} elseif ($uid == 0) {
		// Error!
		return ($_CONFIG['surfbar_max_order'] + 1);
	}

	// Get amount from database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE userid=%s
LIMIT %s",
		array($uid, $_CONFIG['surfbar_max_order']), __FILE__, __LINE__
	);

	// Fetch row
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}
// Generate a validation code for the given id number
function SURFBAR_GENERATE_VALIDATION_CODE ($id, $salt="") {
	global $_CONFIG, $SURFBAR_CACHE;

	// Generate a code until the length matches
	$valCode = "";
	while (strlen($valCode) != $_CONFIG['code_length']) {
		// Is the salt set?
		if (empty($salt)) {
			// Generate random hashed string
			$SURFBAR_CACHE['salt'] = sha1(GEN_PASS(255));
			//DEBUG_LOG(__FUNCTION__.":newSalt=".SURFBAR_GET_SALT()."");
		} else {
			// Use this as salt!
			$SURFBAR_CACHE['salt'] = $salt;
			//DEBUG_LOG(__FUNCTION__.":oldSalt=".SURFBAR_GET_SALT()."");
		}

		// ... and now the validation code
		$valCode = GEN_RANDOM_CODE($_CONFIG['code_length'], sha1(SURFBAR_GET_SALT().":".$id), $GLOBALS['userid']);
		//DEBUG_LOG(__FUNCTION__.":valCode={$valCode}");
	} // END - while

	// Hash it with md5() and salt it with the random string
	$hashedCode = generateHash(md5($valCode), SURFBAR_GET_SALT());

	// Finally encrypt it PGP-like and return it
	$valHashedCode = generatePassString($hashedCode);
	//DEBUG_LOG(__FUNCTION__.":finalValCode={$valHashedCode}");
	return $valHashedCode;
}
// Check validation code
function SURFBAR_CHECK_VALIDATION_CODE ($id, $check, $salt) {
	global $SURFBAR_CACHE;

	// Secure id number
	$id = bigintval($id);

	// Now generate the code again
	$code = SURFBAR_GENERATE_VALIDATION_CODE($id, $salt);

	// Return result of checking hashes and salts
	//DEBUG_LOG(__FUNCTION__.":---".$code."|".$check."---");
	//DEBUG_LOG(__FUNCTION__.":+++".$salt."|".SURFBAR_GET_DATA('last_salt')."+++");
	return (($code == $check) && ($salt == SURFBAR_GET_DATA('last_salt')));
}
// Lockdown the userid/id combination (reload lock)
function SURFBAR_LOCKDOWN_ID ($id) {
	//* DEBUG: */ print "LOCK!");
	///* DEBUG: */ return;
	// Just add it to the database
	SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_locks (userid, url_id) VALUES(%s, %s)",
		array($GLOBALS['userid'], bigintval($id)), __FILE__, __LINE__);

	// Remove the salt from database
	SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_surfbar_salts WHERE url_id=%s AND userid=%s LIMIT 1",
		array(bigintval($id), $GLOBALS['userid']), __FILE__, __LINE__);
}
// Pay points to the user and remove it from the sender
function SURFBAR_PAY_POINTS ($id) {
	global $SURFBAR_CACHE, $_CONFIG;

	// Re-configure ref-system to surfbar levels
	$_CONFIG['db_percents'] = "percent";
	$_CONFIG['db_table']    = "surfbar_reflevels";

	// Remove it from the URL owner
	//DEBUG_LOG(__FUNCTION__.":uid=".SURFBAR_GET_USERID().",costs=".SURFBAR_GET_COSTS()."");
	SUB_POINTS(SURFBAR_GET_USERID(), SURFBAR_GET_COSTS());

	// Book it to the user
	//DEBUG_LOG(__FUNCTION__.":uid=".$GLOBALS['userid'].",reward=".SURFBAR_GET_REWARD()."");
	ADD_POINTS_REFSYSTEM($GLOBALS['userid'], SURFBAR_GET_DATA('reward'));
}
// Updates the statistics of current URL/userid
function SURFBAR_UPDATE_INSERT_STATS_RECORD () {
	global $_CONFIG;

	// Update views_total
	SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_urls SET views_total=views_total+1 WHERE id=%s LIMIT 1",
		array(SURFBAR_GET_ID()), __FILE__, __LINE__);

	// Update the stats entry
	SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_stats SET count=count+1 WHERE userid=%s AND url_id=%s LIMIT 1",
		array($GLOBALS['userid'], SURFBAR_GET_ID()), __FILE__, __LINE__);

	// Was that update okay?
	if (SQL_AFFECTEDROWS() == 0) {
		// No, then insert entry
		SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_stats (userid,url_id,count) VALUES(%s,%s,1)",
			array($GLOBALS['userid'], SURFBAR_GET_ID()), __FILE__, __LINE__);
	} // END - if

	// Update total/daily/weekly/monthly counter
	$_CONFIG['surfbar_total_counter']++;
	$_CONFIG['surfbar_daily_counter']++;
	$_CONFIG['surfbar_weekly_counter']++;
	$_CONFIG['surfbar_monthly_counter']++;

	// Update config as well
	UPDATE_CONFIG(array("surfbar_total_counter", "surfbar_daily_counter", "surfbar_weekly_counter", "surfbar_monthly_counter"), array(1,1,1,1), "+");
}
// Update the salt for validation and statistics
function SURFBAR_UPDATE_SALT_STATS () {
	// Update statistics record
	SURFBAR_UPDATE_INSERT_STATS_RECORD();

	// Simply store the salt from cache away in database...
	SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_salts SET last_salt='%s' WHERE url_id=%s AND userid=%s LIMIT 1",
		array(SURFBAR_GET_SALT(), SURFBAR_GET_ID(), $GLOBALS['userid']), __FILE__, __LINE__);

	// Debug message
	//DEBUG_LOG(__FUNCTION__.":salt=".SURFBAR_GET_SALT().",id=".SURFBAR_GET_ID().",uid=".$GLOBALS['userid']."");

	// Was that okay?
	if (SQL_AFFECTEDROWS() == 0) {
		// Insert missing entry!
		SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_salts (url_id,userid,last_salt) VALUES(%s, %s, '%s')",
			array(SURFBAR_GET_ID(), $GLOBALS['userid'], SURFBAR_GET_SALT()), __FILE__, __LINE__);
	} // END - if

	// Debug message
	//DEBUG_LOG(__FUNCTION__.":affectedRows=".SQL_AFFECTEDROWS()."");

	// Return if the update was okay
	return (SQL_AFFECTEDROWS() == 1);
}
// Check if the reload lock is active for given id
function SURFBAR_CHECK_RELOAD_LOCK ($id) {
	//DEBUG_LOG(__FUNCTION__.":id={$id}");
	// Ask the database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM "._MYSQL_PREFIX."_surfbar_locks
WHERE userid=%s AND url_id=%s AND (UNIX_TIMESTAMP() - ".SURFBAR_GET_DATA('surf_lock').") < UNIX_TIMESTAMP(last_surfed)
ORDER BY last_surfed ASC
LIMIT 1",
		array($GLOBALS['userid'], bigintval($id)), __FILE__, __LINE__
	);

	// Fetch counter
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return check
	//DEBUG_LOG(__FUNCTION__.":cnt={$cnt},".SURFBAR_GET_DATA('surf_lock')."");
	return ($cnt == 1);
}
// Determine which user hash no more points left
function SURFBAR_DETERMINE_DEPLETED_USERIDS() {
	// Init array
	$UIDs = array();

	// Do we have a current user id?
	if (IS_LOGGED_IN()) {
		// Then add this as well
		$UIDs[] = $GLOBALS['userid'];

		// Get all userid except logged in one
		$result = SQL_QUERY_ESC("SELECT userid FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE userid != %s AND status='CONFIRMED'
GROUP BY userid
ORDER BY userid ASC",
			array($GLOBALS['userid']), __FILE__, __LINE__);
	} else {
		// Get all userid
		$result = SQL_QUERY_ESC("SELECT userid FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE status='CONFIRMED'
GROUP BY userid
ORDER BY userid ASC", __FILE__, __LINE__);
	}

	// Load all userid
	while (list($uid) = SQL_FETCHROW($result)) {
		// Get total points
		$points = GET_TOTAL_DATA($uid, "user_points", "points") - GET_TOTAL_DATA($uid, "user_data", "used_points");
		//DEBUG_LOG(__FUNCTION__.":uid={$uid},points={$points}");

		// Shall we add this to ignore?
		if ($points <= 0) {
			// Ignore this one!
			//DEBUG_LOG(__FUNCTION__.":uid={$uid} has depleted points amount!");
			$UIDs[] = $uid;
		} // END - if
	} // END - while

	// Free result
	SQL_FREERESULT($result);

	// Debug message
	//DEBUG_LOG(__FUNCTION__.":UIDs::count=".count($UIDs)." (with own userid=".$GLOBALS['userid'].")");

	// Return result
	return $UIDs;
}
// Determine how many users are Online in surfbar
function SURFBAR_DETERMINE_TOTAL_ONLINE () {
	global $_CONFIG;

	// Count all users in surfbar modue and return the value
	$result = SQL_QUERY_ESC("SELECT COUNT(id) FROM "._MYSQL_PREFIX."_surfbar_stats WHERE (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_online)) <= %s",
		array($_CONFIG['online_timeout']), __FILE__, __LINE__);

	// Fetch count
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}
// Determine next id for surfbar view, always call this before you call other
// getters below this function!!!
function SURFBAR_GET_NEXT_ID ($id = 0) {
	global $SURFBAR_CACHE, $_CONFIG;

	// Default is no id!
	$nextId = 0; $randNum = 0;

	// Is the ID set?
	if ($id == 0) {
		// Prepare some arrays
		$IDs = array();
		$USE = array();
		$ignored = array();

		// Get all id from locks within the timestamp
		$result = SQL_QUERY_ESC("SELECT id, url_id, UNIX_TIMESTAMP(last_surfed)
FROM
	"._MYSQL_PREFIX."_surfbar_locks
WHERE
	userid=%s
ORDER BY
	id ASC", array($GLOBALS['userid']),
			__FILE__, __LINE__);

		// Load all entries
		while (list($id, $url, $last) = SQL_FETCHROW($result)) {
			//DEBUG_LOG(__FUNCTION__.":next - id={$id},url={$url},last={$last}");
			// Skip entries that are too old
			if (($last < (time() - SURFBAR_GET_DATA('surf_lock'))) && (!in_array($url, $ignored))) {
				//DEBUG_LOG(__FUNCTION__.":okay - id={$id},url={$url},last={$last}");
				// Add only if missing or bigger
				if ((!isset($IDs[$url])) || ($IDs[$url] <= $last)) {
					// Add this ID
					//DEBUG_LOG(__FUNCTION__.":ADD - id={$id},url={$url},last={$last}");
					$IDs[$url] = $last;
					$USE[$url] = $id;
				} // END - if
			} else {
				// Ignore these old entries!
				//DEBUG_LOG(__FUNCTION__.":ignore - id={$id},url={$url},last={$last}");
				$ignored[] = $url;
				unset($IDs[$url]);
				unset($USE[$url]);
			}
		} // END - while

		// Free result
		SQL_FREERESULT($result);

		// Shall we add some ids?
		$ADD = "";
		if (count($USE) > 0) {
			$ADD = " AND l.id IN (".implode(",", $USE).")";
		} // END - if

		// Determine depleted user account
		$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS();

		// Count max availabe entries
		$result = SQL_QUERY("SELECT sbu.id AS cnt
FROM "._MYSQL_PREFIX."_surfbar_urls AS sbu
LEFT JOIN "._MYSQL_PREFIX."_payments AS p
ON sbu.payment_id=p.id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_salts AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_locks AS l
ON sbu.id=l.url_id
WHERE sbu.userid NOT IN (".implode(",", $UIDs).") AND sbu.status='CONFIRMED'".$ADD."
GROUP BY sbu.id", __FILE__, __LINE__);

		// Log last query
		//DEBUG_LOG(__FUNCTION__.":lastQuery=".$_CONFIG['db_last_query']."|numRows=".SQL_NUMROWS($result)."|Affected=".SQL_AFFECTEDROWS($result)."");

		// Fetch max rand
		$maxRand = SQL_NUMROWS($result);

		// Free result
		SQL_FREERESULT($result);

		// If more than one URL can be called generate the random number!
		if ($maxRand > 1) {
			// Generate random number
			$randNum = mt_rand(0, $maxRand);
		} // END - if

		// And query the database
		//DEBUG_LOG(__FUNCTION__.":randNum={$randNum},maxRand={$maxRand},surfLock=".SURFBAR_GET_DATA('surf_lock')."");
		$result = SQL_QUERY_ESC("SELECT sbu.id, sbu.userid, sbu.url, sbs.last_salt, sbu.reward, sbu.costs, sbu.views_total, p.time, UNIX_TIMESTAMP(l.last_surfed) AS last_surfed
FROM "._MYSQL_PREFIX."_surfbar_urls AS sbu
LEFT JOIN "._MYSQL_PREFIX."_payments AS p
ON sbu.payment_id=p.id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_salts AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_locks AS l
ON sbu.id=l.url_id
WHERE sbu.userid NOT IN (".implode(",", $UIDs).") AND sbu.status='CONFIRMED'".$ADD."
GROUP BY sbu.id
ORDER BY l.last_surfed ASC, sbu.id ASC
LIMIT %s,1",
			array($randNum), __FILE__, __LINE__
		);
	} else {
		// Get data from specified id number
		$result = SQL_QUERY_ESC("SELECT sbu.id, sbu.userid, sbu.url, sbs.last_salt, sbu.reward, sbu.costs, sbu.views_total, p.time, UNIX_TIMESTAMP(l.last_surfed) AS last_surfed
FROM "._MYSQL_PREFIX."_surfbar_urls AS sbu
LEFT JOIN "._MYSQL_PREFIX."_payments AS p
ON sbu.payment_id=p.id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_salts AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_locks AS l
ON sbu.id=l.url_id
WHERE sbu.userid != %s AND sbu.status='CONFIRMED' AND sbu.id=%s
LIMIT 1",
			array($GLOBALS['userid'], bigintval($id)), __FILE__, __LINE__
		);
	}

	// Is there an id number?
	//DEBUG_LOG(__FUNCTION__.":lastQuery=".$_CONFIG['db_last_query']."|numRows=".SQL_NUMROWS($result)."|Affected=".SQL_AFFECTEDROWS($result)."");
	if (SQL_NUMROWS($result) == 1) {
		// Load/cache data
		//DEBUG_LOG(__FUNCTION__.":count(".count($SURFBAR_CACHE).") - BEFORE");
		$SURFBAR_CACHE = merge_array($SURFBAR_CACHE, SQL_FETCHARRAY($result));
		//DEBUG_LOG(__FUNCTION__.":count(".count($SURFBAR_CACHE).") - AFTER");

		// Is the time there?
		if (is_null($SURFBAR_CACHE['time'])) {
			// Then repair it wit the static!
			//DEBUG_LOG(__FUNCTION__.":time - STATIC!");
			$SURFBAR_CACHE['time'] = $_CONFIG['surfbar_static_time'];
		} // END - if

		// Is the last salt there?
		if (is_null($SURFBAR_CACHE['last_salt'])) {
			// Then repair it wit the static!
			//DEBUG_LOG(__FUNCTION__.":last_salt - FIXED!");
			$SURFBAR_CACHE['last_salt'] = "";
		} // END - if

		// Fix missing last_surfed
		if ((!isset($SURFBAR_CACHE['last_surfed'])) || (is_null($SURFBAR_CACHE['last_surfed']))) {
			// Fix it here
			//DEBUG_LOG(__FUNCTION__.":last_surfed - FIXED!");
			$SURFBAR_CACHE['last_surfed'] = "0";
		} // END - if

		// Get base/fixed reward and costs
		$SURFBAR_CACHE['reward'] = SURFBAR_DETERMINE_REWARD();
		$SURFBAR_CACHE['costs']  = SURFBAR_DETERMINE_COSTS();
		//DEBUG_LOG(__FUNCTION__.":BASE/STATIC - reward=".SURFBAR_GET_REWARD()."|costs=".SURFBAR_GET_COSTS()."");

		// Only in dynamic model add the dynamic bonus!
		if ($_CONFIG['surfbar_pay_model'] == "DYNAMIC") {
			// Calculate dynamic reward/costs and add it
			$SURFBAR_CACHE['reward'] += SURFBAR_CALCULATE_DYNAMIC_REWARD_ADD();
			$SURFBAR_CACHE['costs']  += SURFBAR_CALCULATE_DYNAMIC_COSTS_ADD();
			//DEBUG_LOG(__FUNCTION__.":DYNAMIC+ - reward=".SURFBAR_GET_REWARD()."|costs=".SURFBAR_GET_COSTS()."");
		} // END - if

		// Now get the id
		$nextId = SURFBAR_GET_ID();
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	//DEBUG_LOG(__FUNCTION__.":nextId={$nextId}");
	return $nextId;
}
// ----------------------------------------------------------------------------
// PLEASE DO NOT ADD ANY OTHER FUNCTIONS BELOW THIS LINE ELSE THEY "WRAP" THE
// $SURFBAR_CACHE ARRAY!
// ----------------------------------------------------------------------------
// Private getter for data elements
function SURFBAR_GET_DATA ($element) {
	global $SURFBAR_CACHE;
	//DEBUG_LOG(__FUNCTION__.":element={$element}");

	// Default is null
	$data = null;

	// Is the entry there?
	if (isset($SURFBAR_CACHE[$element])) {
		// Then take it
		$data = $SURFBAR_CACHE[$element];
	} else { // END - if
		print("<pre>");
		print_r($SURFBAR_CACHE);
		debug_print_backtrace();
		die("</pre>");
	}

	// Return result
	//DEBUG_LOG(__FUNCTION__.":element[$element]={$data}");
	return $data;
}
// Getter for reward from cache
function SURFBAR_GET_REWARD () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('reward');
}
// Getter for costs from cache
function SURFBAR_GET_COSTS () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('costs');
}
// Getter for URL from cache
function SURFBAR_GET_URL () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('url');
}
// Getter for salt from cache
function SURFBAR_GET_SALT () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('salt');
}
// Getter for id from cache
function SURFBAR_GET_ID () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('id');
}
// Getter for userid from cache
function SURFBAR_GET_USERID () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('userid');
}
// Getter for user reload locks
function SURFBAR_GET_USER_RELOAD_LOCK () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('user_locks');
}
// Getter for reload time
function SURFBAR_GET_RELOAD_TIME () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('time');
}
//
?>
